/*
 * Created on 2005/01/28
 *
 */
package jnet.hiromicam.msg;

import java.util.ArrayList;
import java.util.Calendar;

import jnet.hiromicam.msg.MsgImpl;
import jnet.hiromicam.properity.SystemProperty;

/**
 * This is an abstract class for Second-Level Information. 
 * In this class, we define the common field of Second-Level Information.
 * We define the information which is exchanged in needed, as Second-Level Information. 
 * For example, Emergencies, information for life, and so on.
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public abstract class AbstractSecondLevelInfo extends MsgImpl {
	/**
	 * This represents tag of the content ID.
	 */
	public static final String CONTENT_ID_TAG = "ContentID";
	
	/**
	 * This represents tag of the content type.
	 */
	public static final String CONTENT_TYPE_TAG = "Content_Type";
	
	/**
	 * This represents tag of the priority.
	 */
	public static final String PRIORITY_TAG = "Priority";
	
	/**
	 * This represents tag of the sender of this content.
	 */
	public static final String GENERATOR_TAG = "Generator";
	
	/**
	 * This represents tag of the time when this content is generated.
	 */
	public static final String GENERATED_TIME_TAG = "Generated_Time";
	
	/**
	 * This represents tag of the source content ID.
	 */
	public static final String SOURCE_CONTENT_ID_TAG = "Source_ContentID";
	
	/**
	 * This represents tag of the data.
	 */
	public static final String CONTENT_TAG = "Content";
	
	/**
	 * This represents the priority level, high.
	 */
	public static final String PRIORITY_HIGH = "high";
	
	/**
	 * This represents the priority level, middle.
	 */
	public static final String PRIORITY_MIDDLE = "middle";
	
	/**
	 * This represents the priority level, low.
	 */
	public static final String PRIORITY_LOW = "low";
	
	/**
	 * the Content ID.
	 */
	protected String contentID;
	
	/**
	 * the Content Type
	 */
	protected String contentType;
	
	/**
	 * the priority of this content.
	 */
	protected String priority;
	
	/**
	 * the time when this content is generated.
	 */
	protected String generatedTime;
	
	/**
	 * the generator IP Address of this content.
	 */
	protected String generator;
	
	/**
	 * the source content ID of this content.
	 */
	protected ArrayList sourceContentID;
	
	/**
	 * This is a default constractor.
	 * 
	 * @param contentID the content ID of this content.
	 * @param contentType the content type.
	 * @param priority the priority of this sender.
	 * @param generatedTime the time when this content was generated.
	 * @param generator the IP Address who generated this content.
	 * @param sourceContentID the source content ID of this content.
	 */
	public AbstractSecondLevelInfo(String contentID,
									String contentType,
									String priority,
									String generatedTime,
									String generator,
									ArrayList sourceContentID) {
		this.msgType = SECOND_LEVEL_INFO;
		this.contentID = contentID;
		this.contentType = contentType;
		this.priority = priority;
		this.generatedTime = generatedTime;
		this.generator = generator;
		this.contentType = contentType;
		if (sourceContentID == null) {
			this.sourceContentID = new ArrayList();
		} else {
			this.sourceContentID = sourceContentID;
		}
	}
	
	/**
	 * This is a constractor for generating new content.
	 * 
	 * @param contentType the content type.
	 * @param priority the priority of this sender.
	 * @param sourceContentID the source content ID of this content.
	 */
	public AbstractSecondLevelInfo(String contentType,
									String priority,
									ArrayList sourceContentID) {
		this.contentID = String.valueOf(SystemProperty.getInstance().getAddr().hashCode())
						  + String.valueOf((int)(Math.random() * 1000));
		this.generatedTime = String.valueOf(Calendar.getInstance().getTimeInMillis());
		this.generator = String.valueOf(SystemProperty.getInstance().getAddr());
		this.contentType = contentType;
		this.priority = priority;
		if (sourceContentID == null) {
			this.sourceContentID = new ArrayList();
		} else {
			this.sourceContentID = sourceContentID;
		}
	}
	
	/**
	 * This method returns the content ID.
	 * 
	 * @return Returns the contentID.
	 */
	public String getContentID() {
		return contentID;
	}
	
	/**
	 * This method returns the content type.
	 * 
	 * @return Returns the contentType.
	 */
	public String getContentType() {
		return contentType;
	}
	
	/**
	 * This method returns the creator of this content.
	 * 
	 * @return Returns the creator.
	 */
	public String getCreator() {
		return generator;
	}
	
	/**
	 * This method returns the priority.
	 * 
	 * @return Returns the priority.
	 */
	public String getPriority() {
		return priority;
	}
	
	/**
	 * This method returns the source content ID.
	 * 
	 * @return Returns the sourceContentID.
	 */
	public ArrayList getSourceContentID() {
		return sourceContentID;
	}
	
	/**
	 * This method returns the time of this content generated.
	 * 
	 * @return Returns the generatedTime.
	 */
	public String getGeneratedTime() {
		return generatedTime;
	}
	
	/**
	 * This method returns the detaial data of this content.
	 * 
	 * @return Returns the data.
	 */
	abstract public String getConcreatMsg();
	
	// This method over ride javadoc of super class.
	public String getMsg() {
		
		String str = "";
		for (int i = 0; i < sourceContentID.size(); i++) {
			str += "\t<" + SOURCE_CONTENT_ID_TAG + ">"
					+ (String)sourceContentID.get(i)
					+ "</" + SOURCE_CONTENT_ID_TAG + ">\n";
		}
			
		return XML_HEADDER + "\n"
				+ "<" + SECOND_LEVEL_INFO_TAG + ">\n"
				+ "\t<" + CONTENT_ID_TAG + ">" + contentID + "</" + CONTENT_ID_TAG + ">\n"
				+ "\t<" + CONTENT_TYPE_TAG + ">" + contentType + "</" + CONTENT_TYPE_TAG + ">\n"
				+ "\t<" + PRIORITY_TAG + ">" + priority + "</" + PRIORITY_TAG + ">\n"
				+ "\t<" + GENERATOR_TAG + ">" + generator + "</" + GENERATOR_TAG + ">\n"
				+ "\t<" + GENERATED_TIME_TAG + ">" + generatedTime + "</" + GENERATED_TIME_TAG + ">\n"
				+ str
				+ getConcreatMsg()
				+ "</" + SECOND_LEVEL_INFO_TAG + ">";
	}

}
